@@ -1,9 +1,10 @@
 import errno
 import os
+import re
 import socket
 import struct
 from struct import pack
-from unittest.mock import ANY, MagicMock, Mock, call, patch
+from unittest.mock import ANY, MagicMock, Mock, call, patch, sentinel
 
 import pytest
 
@@ -53,6 +54,12 @@ class MockSocket:
         self.connected = False
         self.sa = None
 
+    def getsockname(self):
+        return ('127.0.0.1', 1234)
+
+    def getpeername(self):
+        return ('1.2.3.4', 5671)
+
 
 TCP_KEEPIDLE = 4
 TCP_KEEPINTVL = 5
@@ -326,8 +333,11 @@ class test_AbstractTransport:
         self.t._read.return_value = pack('>BHI', 1, 1, 16)
         self.t._read.side_effect = on_read1
         checksum[0] = b'\x13'
-        with pytest.raises(UnexpectedFrame):
+        with pytest.raises(UnexpectedFrame) as ex:
             self.t.read_frame()
+        assert ex.value.code == 505
+        assert ex.value.message == \
+            'Received frame_end 0x13 while expecting 0xce'
 
     def test_read_frame__long(self):
         self.t._read = Mock()
@@ -460,13 +470,13 @@ class test_AbstractTransport_connect:
 
     def test_connect_socket_initialization_fails(self):
         with patch('socket.socket', side_effect=socket.error), \
-             patch('socket.getaddrinfo',
-                   return_value=[
-                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                        '', ('127.0.0.1', 5672)),
-                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                        '', ('127.0.0.2', 5672))
-                   ]):
+            patch('socket.getaddrinfo',
+                  return_value=[
+                      (socket.AF_INET, 1, socket.IPPROTO_TCP,
+                       '', ('127.0.0.1', 5672)),
+                      (socket.AF_INET, 1, socket.IPPROTO_TCP,
+                       '', ('127.0.0.2', 5672))
+                  ]):
             with pytest.raises(socket.error):
                 self.t.connect()
             assert self.t.sock is None and self.t.connected is False
@@ -504,13 +514,13 @@ class test_AbstractTransport_connect:
 
     def test_connect_short_curcuit_on_INET_succeed(self):
         with patch('socket.socket', return_value=MockSocket()), \
-             patch('socket.getaddrinfo',
-                   side_effect=[
-                       [(socket.AF_INET, 1, socket.IPPROTO_TCP,
-                         '', ('127.0.0.1', 5672))],
-                       [(socket.AF_INET6, 1, socket.IPPROTO_TCP,
-                         '', ('::1', 5672))]
-                   ]) as getaddrinfo:
+            patch('socket.getaddrinfo',
+                  side_effect=[
+                      [(socket.AF_INET, 1, socket.IPPROTO_TCP,
+                        '', ('127.0.0.1', 5672))],
+                      [(socket.AF_INET6, 1, socket.IPPROTO_TCP,
+                        '', ('::1', 5672))]
+                  ]) as getaddrinfo:
             self.t.sock = Mock()
             self.t.close()
             self.t.connect()
@@ -542,19 +552,19 @@ class test_AbstractTransport_connect:
 
     def test_connect_getaddrinfo_raises_gaierror_once_recovers(self):
         with patch('socket.socket', return_value=MockSocket()), \
-             patch('socket.getaddrinfo',
-                   side_effect=[
-                       socket.gaierror,
-                       [(socket.AF_INET6, 1, socket.IPPROTO_TCP,
-                         '', ('::1', 5672))]
-                   ]):
+            patch('socket.getaddrinfo',
+                  side_effect=[
+                      socket.gaierror,
+                      [(socket.AF_INET6, 1, socket.IPPROTO_TCP,
+                        '', ('::1', 5672))]
+                  ]):
             self.t.connect()
 
     def test_connect_survives_not_implemented_set_cloexec(self):
         with patch('socket.socket', return_value=MockSocket()), \
-             patch('socket.getaddrinfo',
-                   return_value=[(socket.AF_INET, 1, socket.IPPROTO_TCP,
-                                  '', ('127.0.0.1', 5672))]):
+            patch('socket.getaddrinfo',
+                  return_value=[(socket.AF_INET, 1, socket.IPPROTO_TCP,
+                                 '', ('127.0.0.1', 5672))]):
             with patch('amqp.transport.set_cloexec',
                        side_effect=NotImplementedError) as cloexec_mock:
                 self.t.connect()
@@ -579,6 +589,20 @@ class test_SSLTransport:
         def _init_socket(self, *args):
             pass
 
+    def test_repr_disconnected(self):
+        assert re.fullmatch(
+            r'<SSLTransport: \(disconnected\) at 0x.*>',
+            repr(transport.SSLTransport('host', 3))
+        )
+
+    def test_repr_connected(self):
+        t = transport.SSLTransport('host', 3)
+        t.sock = MockSocket()
+        re.fullmatch(
+            '<SSLTransport: 127.0.0.1:1234 -> 1.2.3.4:5671 at 0x.*>',
+            repr(t)
+        )
+
     @pytest.fixture(autouse=True)
     def setup_transport(self):
         self.t = self.Transport(
@@ -614,14 +638,136 @@ class test_SSLTransport:
             ctx.wrap_socket.assert_called_with(sock, f=1)
 
     def test_wrap_socket_sni(self):
+        # testing default values of _wrap_socket_sni()
         sock = Mock()
-        with patch('ssl.SSLContext.wrap_socket') as mock_ssl_wrap:
-            self.t._wrap_socket_sni(sock)
-            mock_ssl_wrap.assert_called_with(sock=sock,
-                                             server_side=False,
-                                             do_handshake_on_connect=False,
-                                             suppress_ragged_eofs=True,
-                                             server_hostname=None)
+        with patch(
+                'ssl.SSLContext.wrap_socket',
+                return_value=sentinel.WRAPPED_SOCKET) as mock_ssl_wrap:
+            ret = self.t._wrap_socket_sni(sock)
+
+        mock_ssl_wrap.assert_called_with(sock=sock,
+                                         server_side=False,
+                                         do_handshake_on_connect=False,
+                                         suppress_ragged_eofs=True,
+                                         server_hostname=None)
+
+        assert ret == sentinel.WRAPPED_SOCKET
+
+    def test_wrap_socket_sni_certfile(self):
+        # testing _wrap_socket_sni() with parameters certfile and keyfile
+        sock = Mock()
+        with patch(
+            'ssl.SSLContext.wrap_socket',
+            return_value=sentinel.WRAPPED_SOCKET) as mock_ssl_wrap, \
+                patch('ssl.SSLContext.load_cert_chain') as mock_load_cert_chain:
+            ret = self.t._wrap_socket_sni(
+                sock, keyfile=sentinel.KEYFILE, certfile=sentinel.CERTFILE)
+
+        mock_load_cert_chain.assert_called_with(
+            sentinel.CERTFILE, sentinel.KEYFILE)
+        mock_ssl_wrap.assert_called_with(sock=sock,
+                                         server_side=False,
+                                         do_handshake_on_connect=False,
+                                         suppress_ragged_eofs=True,
+                                         server_hostname=None)
+
+        assert ret == sentinel.WRAPPED_SOCKET
+
+    def test_wrap_socket_ca_certs(self):
+        # testing _wrap_socket_sni() with parameter ca_certs
+        sock = Mock()
+        with patch(
+            'ssl.SSLContext.wrap_socket',
+            return_value=sentinel.WRAPPED_SOCKET
+        ) as mock_ssl_wrap, patch(
+            'ssl.SSLContext.load_verify_locations'
+        ) as mock_load_verify_locations:
+            ret = self.t._wrap_socket_sni(sock, ca_certs=sentinel.CA_CERTS)
+
+        mock_load_verify_locations.assert_called_with(sentinel.CA_CERTS)
+        mock_ssl_wrap.assert_called_with(sock=sock,
+                                         server_side=False,
+                                         do_handshake_on_connect=False,
+                                         suppress_ragged_eofs=True,
+                                         server_hostname=None)
+
+        assert ret == sentinel.WRAPPED_SOCKET
+
+    def test_wrap_socket_ciphers(self):
+        # testing _wrap_socket_sni() with parameter ciphers
+        sock = Mock()
+        with patch(
+            'ssl.SSLContext.wrap_socket',
+            return_value=sentinel.WRAPPED_SOCKET) as mock_ssl_wrap, \
+                patch('ssl.SSLContext.set_ciphers') as mock_set_ciphers:
+            ret = self.t._wrap_socket_sni(sock, ciphers=sentinel.CIPHERS)
+
+        mock_set_ciphers.assert_called_with(sentinel.CIPHERS)
+        mock_ssl_wrap.assert_called_with(sock=sock,
+                                         server_side=False,
+                                         do_handshake_on_connect=False,
+                                         suppress_ragged_eofs=True,
+                                         server_hostname=None)
+        assert ret == sentinel.WRAPPED_SOCKET
+
+    def test_wrap_socket_sni_cert_reqs(self):
+        # testing _wrap_socket_sni() with parameter cert_reqs
+        sock = Mock()
+        with patch('ssl.SSLContext') as mock_ssl_context_class:
+            wrap_socket_method_mock = mock_ssl_context_class().wrap_socket
+            wrap_socket_method_mock.return_value = sentinel.WRAPPED_SOCKET
+            ret = self.t._wrap_socket_sni(sock, cert_reqs=sentinel.CERT_REQS)
+
+        wrap_socket_method_mock.assert_called_with(
+            sock=sock,
+            server_side=False,
+            do_handshake_on_connect=False,
+            suppress_ragged_eofs=True,
+            server_hostname=None
+        )
+        assert mock_ssl_context_class().check_hostname is True
+        assert ret == sentinel.WRAPPED_SOCKET
+
+    def test_wrap_socket_sni_setting_sni_header(self):
+        # testing _wrap_socket_sni() with setting SNI header
+        sock = Mock()
+        with patch('ssl.SSLContext') as mock_ssl_context_class, \
+                patch('ssl.HAS_SNI', new=True):
+            # SSL module supports SNI
+            wrap_socket_method_mock = mock_ssl_context_class().wrap_socket
+            wrap_socket_method_mock.return_value = sentinel.WRAPPED_SOCKET
+            ret = self.t._wrap_socket_sni(
+                sock, cert_reqs=sentinel.CERT_REQS,
+                server_hostname=sentinel.SERVER_HOSTNAME
+            )
+        wrap_socket_method_mock.assert_called_with(
+            sock=sock,
+            server_side=False,
+            do_handshake_on_connect=False,
+            suppress_ragged_eofs=True,
+            server_hostname=sentinel.SERVER_HOSTNAME
+        )
+        assert mock_ssl_context_class().verify_mode == sentinel.CERT_REQS
+        assert ret == sentinel.WRAPPED_SOCKET
+
+        with patch('ssl.SSLContext') as mock_ssl_context_class, \
+                patch('ssl.HAS_SNI', new=False):
+            # SSL module does not support SNI
+            wrap_socket_method_mock = mock_ssl_context_class().wrap_socket
+            wrap_socket_method_mock.return_value = sentinel.WRAPPED_SOCKET
+            ret = self.t._wrap_socket_sni(
+                sock, cert_reqs=sentinel.CERT_REQS,
+                server_hostname=sentinel.SERVER_HOSTNAME
+            )
+        wrap_socket_method_mock.assert_called_with(
+            sock=sock,
+            server_side=False,
+            do_handshake_on_connect=False,
+            suppress_ragged_eofs=True,
+            server_hostname=sentinel.SERVER_HOSTNAME
+        )
+        assert mock_ssl_context_class().verify_mode != sentinel.CERT_REQS
+        assert ret == sentinel.WRAPPED_SOCKET
 
     def test_shutdown_transport(self):
         self.t.sock = None
@@ -686,6 +832,20 @@ class test_TCPTransport:
         def _init_socket(self, *args):
             pass
 
+    def test_repr_disconnected(self):
+        assert re.fullmatch(
+            r'<TCPTransport: \(disconnected\) at 0x.*>',
+            repr(transport.TCPTransport('host', 3))
+        )
+
+    def test_repr_connected(self):
+        t = transport.SSLTransport('host', 3)
+        t.sock = MockSocket()
+        re.fullmatch(
+            '<TCPTransport: 127.0.0.1:1234 -> 1.2.3.4:5671 at 0x.*>',
+            repr(t)
+        )
+
     @pytest.fixture(autouse=True)
     def setup_transport(self):
         self.t = self.Transport('host', 3)
